@@ -6,10 +6,12 @@
 use std::iter::Iterator;
 use std::ops::Range;
 
-use crate::parsing::{Scope, ScopeStack, BasicScopeStackOp, ScopeStackOp, MatchPower, ATOM_LEN_BITS};
 use super::selector::ScopeSelector;
-use super::theme::{Theme, ThemeItem};
 use super::style::{Color, FontStyle, Style, StyleModifier};
+use super::theme::{Theme, ThemeItem};
+use crate::parsing::{
+    BasicScopeStackOp, MatchPower, Scope, ScopeStack, ScopeStackOp, ATOM_LEN_BITS,
+};
 
 /// Basically a wrapper around a [`Theme`] preparing it to be used for highlighting.
 ///
@@ -90,7 +92,7 @@ pub struct RangedHighlightIterator<'a, 'b> {
 /// [`Style`]: struct.Style.html
 #[derive(Debug)]
 pub struct HighlightIterator<'a, 'b> {
-    ranged_iterator: RangedHighlightIterator<'a, 'b>
+    ranged_iterator: RangedHighlightIterator<'a, 'b>,
 }
 
 impl HighlightState {
@@ -121,11 +123,12 @@ impl HighlightState {
 }
 
 impl<'a, 'b> RangedHighlightIterator<'a, 'b> {
-    pub fn new(state: &'a mut HighlightState,
-               changes: &'a [(usize, ScopeStackOp)],
-               text: &'b str,
-               highlighter: &'a Highlighter<'_>)
-               -> RangedHighlightIterator<'a, 'b> {
+    pub fn new(
+        state: &'a mut HighlightState,
+        changes: &'a [(usize, ScopeStackOp)],
+        text: &'b str,
+        highlighter: &'a Highlighter<'_>,
+    ) -> RangedHighlightIterator<'a, 'b> {
         RangedHighlightIterator {
             index: 0,
             pos: 0,
@@ -154,34 +157,44 @@ impl<'a, 'b> Iterator for RangedHighlightIterator<'a, 'b> {
         // println!("{} - {:?}   {}:{}", self.index, self.pos, self.state.path.len(), self.state.styles.len());
         let style = *self.state.styles.last().unwrap_or(&Style::default());
         let text = &self.text[self.pos..end];
-        let range = Range { start: self.pos, end };
+        let range = Range {
+            start: self.pos,
+            end,
+        };
         {
             // closures mess with the borrow checker's ability to see different struct fields
             let m_path = &mut self.state.path;
             let m_styles = &mut self.state.styles;
             let m_caches = &mut self.state.single_caches;
             let highlighter = &self.highlighter;
-            m_path.apply_with_hook(&command, |op, cur_stack| {
-                // println!("{:?} - {:?}", op, cur_stack);
-                match op {
-                    BasicScopeStackOp::Push(_) => {
-                        // we can push multiple times so this might have changed
-                        let new_cache = {
-                            if let Some(prev_cache) = m_caches.last() {
-                                highlighter.update_single_cache_for_push(prev_cache, cur_stack)
-                            } else {
-                                highlighter.update_single_cache_for_push(&ScoredStyle::from_style(highlighter.get_default()), cur_stack)
-                            }
-                        };
-                        m_styles.push(highlighter.finalize_style_with_multis(&new_cache, cur_stack));
-                        m_caches.push(new_cache);
+            m_path
+                .apply_with_hook(&command, |op, cur_stack| {
+                    // println!("{:?} - {:?}", op, cur_stack);
+                    match op {
+                        BasicScopeStackOp::Push(_) => {
+                            // we can push multiple times so this might have changed
+                            let new_cache = {
+                                if let Some(prev_cache) = m_caches.last() {
+                                    highlighter.update_single_cache_for_push(prev_cache, cur_stack)
+                                } else {
+                                    highlighter.update_single_cache_for_push(
+                                        &ScoredStyle::from_style(highlighter.get_default()),
+                                        cur_stack,
+                                    )
+                                }
+                            };
+                            m_styles.push(
+                                highlighter.finalize_style_with_multis(&new_cache, cur_stack),
+                            );
+                            m_caches.push(new_cache);
+                        }
+                        BasicScopeStackOp::Pop => {
+                            m_styles.pop();
+                            m_caches.pop();
+                        }
                     }
-                    BasicScopeStackOp::Pop => {
-                        m_styles.pop();
-                        m_caches.pop();
-                    }
-                }
-            });
+                })
+                .ok()?;
         }
         self.pos = end;
         self.index += 1;
@@ -193,21 +206,22 @@ impl<'a, 'b> Iterator for RangedHighlightIterator<'a, 'b> {
     }
 }
 impl<'a, 'b> HighlightIterator<'a, 'b> {
-    pub fn new(state: &'a mut HighlightState,
-               changes: &'a [(usize, ScopeStackOp)],
-               text: &'b str,
-               highlighter: &'a Highlighter<'_>)
-        -> HighlightIterator<'a, 'b> {
-            HighlightIterator {
-                ranged_iterator: RangedHighlightIterator {
-                    index: 0,
-                    pos: 0,
-                    changes,
-                    text,
-                    highlighter,
-                    state
-                }
-            }
+    pub fn new(
+        state: &'a mut HighlightState,
+        changes: &'a [(usize, ScopeStackOp)],
+        text: &'b str,
+        highlighter: &'a Highlighter<'_>,
+    ) -> HighlightIterator<'a, 'b> {
+        HighlightIterator {
+            ranged_iterator: RangedHighlightIterator {
+                index: 0,
+                pos: 0,
+                changes,
+                text,
+                highlighter,
+                state,
+            },
+        }
     }
 }
 
@@ -299,9 +313,13 @@ impl<'a> Highlighter<'a> {
         let mut new_style = cur.clone();
 
         let last_scope = path[path.len() - 1];
-        for &(scope, ref modif) in self.single_selectors.iter().filter(|a| a.0.is_prefix_of(last_scope)) {
-            let single_score = f64::from(scope.len()) *
-                               f64::from(ATOM_LEN_BITS * ((path.len() - 1) as u16)).exp2();
+        for &(scope, ref modif) in self
+            .single_selectors
+            .iter()
+            .filter(|a| a.0.is_prefix_of(last_scope))
+        {
+            let single_score = f64::from(scope.len())
+                * f64::from(ATOM_LEN_BITS * ((path.len() - 1) as u16)).exp2();
             new_style.apply(modif, MatchPower(single_score));
         }
 
@@ -311,9 +329,10 @@ impl<'a> Highlighter<'a> {
     fn finalize_style_with_multis(&self, cur: &ScoredStyle, path: &[Scope]) -> Style {
         let mut new_style = cur.clone();
 
-        let mult_iter = self.multi_selectors
+        let mult_iter = self
+            .multi_selectors
             .iter()
-            .filter_map(|&(ref sel, ref style)| sel.does_match(path).map(|score| (score, style)));
+            .filter_map(|(sel, style)| sel.does_match(path).map(|score| (score, style)));
         for (score, modif) in mult_iter {
             new_style.apply(modif, score);
         }
@@ -328,7 +347,7 @@ impl<'a> Highlighter<'a> {
     pub fn style_for_stack(&self, stack: &[Scope]) -> Style {
         let mut single_cache = ScoredStyle::from_style(self.get_default());
         for i in 0..stack.len() {
-            single_cache = self.update_single_cache_for_push(&single_cache, &stack[0..i+1]);
+            single_cache = self.update_single_cache_for_push(&single_cache, &stack[0..i + 1]);
         }
         self.finalize_style_with_multis(&single_cache, stack)
     }
@@ -345,18 +364,14 @@ impl<'a> Highlighter<'a> {
     /// [`StyleModifier`]: struct.StyleModifier.html
     /// [`style_for_stack`]: #method.style_for_stack
     pub fn style_mod_for_stack(&self, path: &[Scope]) -> StyleModifier {
-        let mut matching_items : Vec<(MatchPower, &ThemeItem)> = self.theme
+        let mut matching_items: Vec<(MatchPower, &ThemeItem)> = self
+            .theme
             .scopes
             .iter()
-            .filter_map(|item| {
-                item.scope
-                    .does_match(path)
-                    .map(|score| (score, item))
-            })
+            .filter_map(|item| item.scope.does_match(path).map(|score| (score, item)))
             .collect();
         matching_items.sort_by_key(|&(score, _)| score);
-        let sorted = matching_items.iter()
-            .map(|(_, item)| item);
+        let sorted = matching_items.iter().map(|(_, item)| item);
 
         let mut modifier = StyleModifier {
             background: None,
@@ -370,12 +385,12 @@ impl<'a> Highlighter<'a> {
     }
 }
 
-#[cfg(all(feature = "assets", feature = "parsing", any(feature = "dump-load", feature = "dump-load-rs")))]
+#[cfg(all(feature = "default-syntaxes", feature = "default-themes"))]
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::highlighting::{ThemeSet, Style, Color, FontStyle};
-    use crate::parsing::{ SyntaxSet, ScopeStack, ParseState};
+    use crate::highlighting::{Color, FontStyle, Style, ThemeSet};
+    use crate::parsing::{ParseState, ScopeStack, SyntaxSet};
 
     #[test]
     fn can_parse() {
@@ -389,35 +404,40 @@ mod tests {
 
         let mut highlight_state = HighlightState::new(&highlighter, ScopeStack::new());
         let line = "module Bob::Wow::Troll::Five; 5; end";
-        let ops = state.parse_line(line, &ps).unwrap();
+        let ops = state.parse_line(line, &ps).expect("#[cfg(test)]");
         let iter = HighlightIterator::new(&mut highlight_state, &ops[..], line, &highlighter);
         let regions: Vec<(Style, &str)> = iter.collect();
         // println!("{:#?}", regions);
-        assert_eq!(regions[11],
-                   (Style {
-                       foreground: Color {
-                           r: 208,
-                           g: 135,
-                           b: 112,
-                           a: 0xFF,
-                       },
-                       background: Color {
-                           r: 43,
-                           g: 48,
-                           b: 59,
-                           a: 0xFF,
-                       },
-                       font_style: FontStyle::empty(),
-                   },
-                    "5"));
+        assert_eq!(
+            regions[11],
+            (
+                Style {
+                    foreground: Color {
+                        r: 208,
+                        g: 135,
+                        b: 112,
+                        a: 0xFF,
+                    },
+                    background: Color {
+                        r: 43,
+                        g: 48,
+                        b: 59,
+                        a: 0xFF,
+                    },
+                    font_style: FontStyle::empty(),
+                },
+                "5"
+            )
+        );
     }
 
     #[test]
     fn can_parse_with_highlight_state_from_cache() {
         let ps = SyntaxSet::load_from_folder("testdata/Packages").unwrap();
         let mut state = {
-            let syntax = ps.find_syntax_by_scope(
-                Scope::new("source.python").unwrap()).unwrap();
+            let syntax = ps
+                .find_syntax_by_scope(Scope::new("source.python").unwrap())
+                .unwrap();
             ParseState::new(syntax)
         };
         let ts = ThemeSet::load_defaults();
@@ -426,7 +446,7 @@ mod tests {
         // We start by parsing a python multiline-comment: """
         let mut highlight_state = HighlightState::new(&highlighter, ScopeStack::new());
         let line = r#"""""#;
-        let ops = state.parse_line(line, &ps).unwrap();
+        let ops = state.parse_line(line, &ps).expect("#[cfg(test)]");
         let iter = HighlightIterator::new(&mut highlight_state, &ops[..], line, &highlighter);
         assert_eq!(1, iter.count());
         let path = highlight_state.path;
@@ -434,40 +454,59 @@ mod tests {
         // We then parse the next line with a highlight state built from the previous state
         let mut highlight_state = HighlightState::new(&highlighter, path);
         let line = "multiline comment";
-        let ops = state.parse_line(line, &ps).unwrap();
+        let ops = state.parse_line(line, &ps).expect("#[cfg(test)]");
         let iter = HighlightIterator::new(&mut highlight_state, &ops[..], line, &highlighter);
         let regions: Vec<(Style, &str)> = iter.collect();
 
         // We expect the line to be styled as a comment.
-        assert_eq!(regions[0],
-                   (Style {
-                       foreground: Color {
-                           // (Comment: #65737E)
-                           r: 101,
-                           g: 115,
-                           b: 126,
-                           a: 0xFF,
-                       },
-                       background: Color {
-                           r: 43,
-                           g: 48,
-                           b: 59,
-                           a: 0xFF,
-                       },
-                       font_style: FontStyle::empty(),
-                   },
-                    "multiline comment"));
+        assert_eq!(
+            regions[0],
+            (
+                Style {
+                    foreground: Color {
+                        // (Comment: #65737E)
+                        r: 101,
+                        g: 115,
+                        b: 126,
+                        a: 0xFF,
+                    },
+                    background: Color {
+                        r: 43,
+                        g: 48,
+                        b: 59,
+                        a: 0xFF,
+                    },
+                    font_style: FontStyle::empty(),
+                },
+                "multiline comment"
+            )
+        );
     }
 
     // see issues #133 and #203, this test tests the fixes for those issues
     #[test]
     fn tricky_cases() {
+        use crate::highlighting::{ScopeSelectors, ThemeSettings};
         use crate::parsing::ScopeStack;
         use std::str::FromStr;
-        use crate::highlighting::{ThemeSettings, ScopeSelectors};
-        let c1 = Color { r: 1, g: 1, b: 1, a: 255 };
-        let c2 = Color { r: 2, g: 2, b: 2, a: 255 };
-        let def_bg = Color { r: 255, g: 255, b: 255, a: 255 };
+        let c1 = Color {
+            r: 1,
+            g: 1,
+            b: 1,
+            a: 255,
+        };
+        let c2 = Color {
+            r: 2,
+            g: 2,
+            b: 2,
+            a: 255,
+        };
+        let def_bg = Color {
+            r: 255,
+            g: 255,
+            b: 255,
+            a: 255,
+        };
         let test_color_scheme = Theme {
             name: None,
             author: None,
@@ -523,17 +562,46 @@ mod tests {
         let regions: Vec<Style> = iter.map(|(s, _)| s).collect();
 
         // println!("{:#?}", regions);
-        assert_eq!(regions, vec![
-            Style { foreground: c1, background: c1, font_style: FontStyle::ITALIC },
-            Style { foreground: c1, background: def_bg, font_style: FontStyle::ITALIC },
-            Style { foreground: c1, background: c1, font_style: FontStyle::ITALIC },
-        ]);
+        assert_eq!(
+            regions,
+            vec![
+                Style {
+                    foreground: c1,
+                    background: c1,
+                    font_style: FontStyle::ITALIC
+                },
+                Style {
+                    foreground: c1,
+                    background: def_bg,
+                    font_style: FontStyle::ITALIC
+                },
+                Style {
+                    foreground: c1,
+                    background: c1,
+                    font_style: FontStyle::ITALIC
+                },
+            ]
+        );
 
         let full_stack = ScopeStack::from_str("comment.line.rs keyword.control.rs").unwrap();
         let full_style = highlighter.style_for_stack(full_stack.as_slice());
-        assert_eq!(full_style, Style { foreground: c1, background: def_bg, font_style: FontStyle::ITALIC });
+        assert_eq!(
+            full_style,
+            Style {
+                foreground: c1,
+                background: def_bg,
+                font_style: FontStyle::ITALIC
+            }
+        );
         let full_mod = highlighter.style_mod_for_stack(full_stack.as_slice());
-        assert_eq!(full_mod, StyleModifier { foreground: Some(c1), background: None, font_style: Some(FontStyle::ITALIC) });
+        assert_eq!(
+            full_mod,
+            StyleModifier {
+                foreground: Some(c1),
+                background: None,
+                font_style: Some(FontStyle::ITALIC)
+            }
+        );
     }
 
     #[test]
@@ -548,26 +616,31 @@ mod tests {
 
         let mut highlight_state = HighlightState::new(&highlighter, ScopeStack::new());
         let line = "module Bob::Wow::Troll::Five; 5; end";
-        let ops = state.parse_line(line, &ps).unwrap();
+        let ops = state.parse_line(line, &ps).expect("#[cfg(test)]");
         let iter = RangedHighlightIterator::new(&mut highlight_state, &ops[..], line, &highlighter);
         let regions: Vec<(Style, &str, Range<usize>)> = iter.collect();
         // println!("{:#?}", regions);
-        assert_eq!(regions[11],
-                   (Style {
-                       foreground: Color {
-                           r: 208,
-                           g: 135,
-                           b: 112,
-                           a: 0xFF,
-                       },
-                       background: Color {
-                           r: 43,
-                           g: 48,
-                           b: 59,
-                           a: 0xFF,
-                       },
-                       font_style: FontStyle::empty(),
-                   },
-                    "5", Range { start: 30, end: 31 }));
+        assert_eq!(
+            regions[11],
+            (
+                Style {
+                    foreground: Color {
+                        r: 208,
+                        g: 135,
+                        b: 112,
+                        a: 0xFF,
+                    },
+                    background: Color {
+                        r: 43,
+                        g: 48,
+                        b: 59,
+                        a: 0xFF,
+                    },
+                    font_style: FontStyle::empty(),
+                },
+                "5",
+                Range { start: 30, end: 31 }
+            )
+        );
     }
 }
