@@ -10,16 +10,32 @@
 // See https://github.com/rust-lang/rust/blob/1.54.0/library/core/src/hash/mod.rs#L717-L725
 #![allow(clippy::mutable_key_type)]
 
-use super::syntax_definition::*;
-use super::scope::*;
 use super::regex::Region;
-use std::usize;
-use std::collections::HashMap;
-use std::i32;
-use std::hash::BuildHasherDefault;
+use super::scope::*;
+use super::syntax_definition::*;
+use crate::parsing::syntax_definition::ContextId;
+use crate::parsing::syntax_set::{SyntaxReference, SyntaxSet};
 use fnv::FnvHasher;
-use crate::parsing::syntax_set::{SyntaxSet, SyntaxReference};
-use crate::CrashError;
+use std::collections::HashMap;
+use std::hash::BuildHasherDefault;
+use std::i32;
+use std::usize;
+
+/// Errors that can occur while parsing.
+#[derive(Debug, thiserror::Error)]
+#[non_exhaustive]
+pub enum ParsingError {
+    #[error("Somehow main context was popped from the stack")]
+    MissingMainContext,
+    /// A context is missing. Usually caused by a syntax referencing a another
+    /// syntax that is not known to syntect. See e.g. <https://github.com/trishume/syntect/issues/421>
+    #[error("Missing context with ID '{0:?}'")]
+    MissingContext(ContextId),
+    #[error("Bad index to match_at: {0}")]
+    BadMatchIndex(usize),
+    #[error("Tried to use a ContextReference that has not bee resolved yet: {0:?}")]
+    UnresolvedContextReference(ContextReference),
+}
 
 /// Keeps the current parser state (the internal syntax interpreter stack) between lines of parsing.
 ///
@@ -165,7 +181,7 @@ impl ParseState {
     /// main context of the syntax
     pub fn new(syntax: &SyntaxReference) -> ParseState {
         let start_state = StateLevel {
-            context: syntax.context_ids["__start"],
+            context: syntax.context_ids()["__start"],
             prototypes: Vec::new(),
             captures: None,
         };
@@ -199,15 +215,16 @@ impl ParseState {
         &mut self,
         line: &str,
         syntax_set: &SyntaxSet,
-    ) -> Result<Vec<(usize, ScopeStackOp)>, CrashError> {
-        assert!(!self.stack.is_empty(),
-                "Somehow main context was popped from the stack");
+    ) -> Result<Vec<(usize, ScopeStackOp)>, ParsingError> {
+        if self.stack.is_empty() {
+            return Err(ParsingError::MissingMainContext);
+        }
         let mut match_start = 0;
         let mut res = Vec::new();
 
         if self.first_line {
             let cur_level = &self.stack[self.stack.len() - 1];
-            let context = syntax_set.get_context(&cur_level.context);
+            let context = syntax_set.get_context(&cur_level.context)?;
             if !context.meta_content_scope.is_empty() {
                 res.push((0, ScopeStackOp::Push(context.meta_content_scope[0])));
             }
@@ -227,7 +244,7 @@ impl ParseState {
             &mut search_cache,
             &mut regions,
             &mut non_consuming_push_at,
-            &mut res
+            &mut res,
         )? {}
 
         Ok(res)
@@ -243,20 +260,32 @@ impl ParseState {
         regions: &mut Region,
         non_consuming_push_at: &mut (usize, usize),
         ops: &mut Vec<(usize, ScopeStackOp)>,
-    ) -> Result<bool, CrashError> {
+    ) -> Result<bool, ParsingError> {
         let check_pop_loop = {
             let (pos, stack_depth) = *non_consuming_push_at;
             pos == *start && stack_depth == self.stack.len()
         };
 
         // Trim proto_starts that are no longer valid
-        while self.proto_starts.last().map(|start| *start >= self.stack.len()).unwrap_or(false) {
+        while self
+            .proto_starts
+            .last()
+            .map(|start| *start >= self.stack.len())
+            .unwrap_or(false)
+        {
             self.proto_starts.pop();
         }
 
-        let best_match = self.find_best_match(line, *start, syntax_set, search_cache, regions, check_pop_loop)?;
+        let best_match = self.find_best_match(
+            line,
+            *start,
+            syntax_set,
+            search_cache,
+            regions,
+            check_pop_loop,
+        )?;
 
-        Ok(if let Some(reg_match) = best_match {
+        if let Some(reg_match) = best_match {
             if reg_match.would_loop {
                 // A push that doesn't consume anything (a regex that resulted
                 // in an empty match at the current position) can not be
@@ -306,14 +335,14 @@ impl ParseState {
 
             let level_context = {
                 let id = &self.stack[self.stack.len() - 1].context;
-                syntax_set.get_context(id)
+                syntax_set.get_context(id)?
             };
             self.exec_pattern(line, &reg_match, level_context, syntax_set, ops)?;
 
-            true
+            Ok(true)
         } else {
-            false
-        })
+            Ok(false)
+        }
     }
 
     fn find_best_match<'a>(
@@ -324,9 +353,9 @@ impl ParseState {
         search_cache: &mut SearchCache,
         regions: &mut Region,
         check_pop_loop: bool,
-    ) -> Result<Option<RegexMatch<'a>>, CrashError> {
+    ) -> Result<Option<RegexMatch<'a>>, ParsingError> {
         let cur_level = &self.stack[self.stack.len() - 1];
-        let context = syntax_set.get_context(&cur_level.context);
+        let context = syntax_set.get_context(&cur_level.context)?;
         let prototype = if let Some(ref p) = context.prototype {
             Some(p)
         } else {
@@ -337,9 +366,14 @@ impl ParseState {
         let context_chain = {
             let proto_start = self.proto_starts.last().cloned().unwrap_or(0);
             // Sublime applies with_prototypes from bottom to top
-            let with_prototypes = self.stack[proto_start..].iter().flat_map(|lvl| lvl.prototypes.iter().map(move |ctx| (true, ctx, lvl.captures.as_ref())));
+            let with_prototypes = self.stack[proto_start..].iter().flat_map(|lvl| {
+                lvl.prototypes
+                    .iter()
+                    .map(move |ctx| (true, ctx, lvl.captures.as_ref()))
+            });
             let cur_prototype = prototype.into_iter().map(|ctx| (false, ctx, None));
-            let cur_context = Some((false, &cur_level.context, cur_level.captures.as_ref())).into_iter();
+            let cur_context =
+                Some((false, &cur_level.context, cur_level.captures.as_ref())).into_iter();
             with_prototypes.chain(cur_prototype).chain(cur_context)
         };
 
@@ -351,12 +385,12 @@ impl ParseState {
         let mut pop_would_loop = false;
 
         for (from_with_proto, ctx, captures) in context_chain {
-            for (pat_context, pat_index) in context_iter(syntax_set, syntax_set.get_context(ctx)) {
+            for (pat_context, pat_index) in context_iter(syntax_set, syntax_set.get_context(ctx)?) {
                 let match_pat = pat_context.match_at(pat_index)?;
 
-                if let Some(match_region) = self.search(
-                    line, start, match_pat, captures, search_cache, regions
-                ) {
+                if let Some(match_region) =
+                    self.search(line, start, match_pat, captures, search_cache, regions)
+                {
                     let (match_start, match_end) = match_region.pos(0).unwrap();
 
                     // println!("matched pattern {:?} at start {} end {}", match_pat.regex_str, match_start, match_end);
@@ -395,13 +429,14 @@ impl ParseState {
         Ok(best_match)
     }
 
-    fn search(&self,
-              line: &str,
-              start: usize,
-              match_pat: &MatchPattern,
-              captures: Option<&(Region, String)>,
-              search_cache: &mut SearchCache,
-              regions: &mut Region,
+    fn search(
+        &self,
+        line: &str,
+        start: usize,
+        match_pat: &MatchPattern,
+        captures: Option<&(Region, String)>,
+        search_cache: &mut SearchCache,
+        regions: &mut Region,
     ) -> Option<Region> {
         // println!("{} - {:?} - {:?}", match_pat.regex_str, match_pat.has_captures, cur_level.captures.is_some());
         let match_ptr = match_pat as *const MatchPattern;
@@ -422,7 +457,7 @@ impl ParseState {
 
         let (matched, can_cache) = match (match_pat.has_captures, captures) {
             (true, Some(captures)) => {
-                let &(ref region, ref s) = captures;
+                let (region, s) = captures;
                 let regex = match_pat.regex_with_refs(region, s);
                 let matched = regex.search(line, start, line.len(), Some(regions));
                 (matched, false)
@@ -462,13 +497,20 @@ impl ParseState {
         level_context: &'a Context,
         syntax_set: &'a SyntaxSet,
         ops: &mut Vec<(usize, ScopeStackOp)>,
-    ) -> Result<bool, CrashError> {
+    ) -> Result<bool, ParsingError> {
         let (match_start, match_end) = reg_match.regions.pos(0).unwrap();
         let context = reg_match.context;
         let pat = context.match_at(reg_match.pat_index)?;
         // println!("running pattern {:?} on '{}' at {}, operation {:?}", pat.regex_str, line, match_start, pat.operation);
 
-        self.push_meta_ops(true, match_start, level_context, &pat.operation, syntax_set, ops)?;
+        self.push_meta_ops(
+            true,
+            match_start,
+            level_context,
+            &pat.operation,
+            syntax_set,
+            ops,
+        )?;
         for s in &pat.scope {
             // println!("pushing {:?} at {}", s, match_start);
             ops.push((match_start, ScopeStackOp::Push(*s)));
@@ -486,8 +528,10 @@ impl ParseState {
                     }
                     // println!("capture {:?} at {:?}-{:?}", scopes[0], cap_start, cap_end);
                     for scope in scopes.iter() {
-                        map.push(((cap_start, -((cap_end - cap_start) as i32)),
-                                  ScopeStackOp::Push(*scope)));
+                        map.push((
+                            (cap_start, -((cap_end - cap_start) as i32)),
+                            ScopeStackOp::Push(*scope),
+                        ));
                     }
                     map.push(((cap_end, i32::MIN), ScopeStackOp::Pop(scopes.len())));
                 }
@@ -501,20 +545,27 @@ impl ParseState {
             // println!("popping at {}", match_end);
             ops.push((match_end, ScopeStackOp::Pop(pat.scope.len())));
         }
-        self.push_meta_ops(false, match_end, &*level_context, &pat.operation, syntax_set, ops)?;
+        self.push_meta_ops(
+            false,
+            match_end,
+            level_context,
+            &pat.operation,
+            syntax_set,
+            ops,
+        )?;
 
-        Ok(self.perform_op(line, &reg_match.regions, pat, syntax_set))
+        self.perform_op(line, &reg_match.regions, pat, syntax_set)
     }
 
-    fn push_meta_ops<'a>(
+    fn push_meta_ops(
         &self,
         initial: bool,
         index: usize,
         cur_context: &Context,
         match_op: &MatchOperation,
-        syntax_set: &'a SyntaxSet,
+        syntax_set: &SyntaxSet,
         ops: &mut Vec<(usize, ScopeStackOp)>,
-    ) -> Result<(), CrashError> {
+    ) -> Result<(), ParsingError> {
         // println!("metas ops for {:?}, initial: {}",
         //          match_op,
         //          initial);
@@ -531,20 +582,19 @@ impl ParseState {
                 }
 
                 // cleared scopes are restored after the scopes from match pattern that invoked the pop are applied
-                if !initial && cur_context.clear_scopes != None {
-                    ops.push((index, ScopeStackOp::Restore));
+                if !initial && cur_context.clear_scopes.is_some() {
+                    ops.push((index, ScopeStackOp::Restore))
                 }
-            },
+            }
             // for some reason the ST3 behaviour of set is convoluted and is inconsistent with the docs and other ops
             // - the meta_content_scope of the current context is applied to the matched thing, unlike pop
             // - the clear_scopes are applied after the matched token, unlike push
             // - the interaction with meta scopes means that the token has the meta scopes of both the current scope and the new scope.
-            MatchOperation::Push(ref context_refs) |
-            MatchOperation::Set(ref context_refs) => {
+            MatchOperation::Push(ref context_refs) | MatchOperation::Set(ref context_refs) => {
                 let is_set = matches!(*match_op, MatchOperation::Set(_));
                 // a match pattern that "set"s keeps the meta_content_scope and meta_scope from the previous context
                 if initial {
-                    if is_set && cur_context.clear_scopes != None {
+                    if is_set && cur_context.clear_scopes.is_some() {
                         // cleared scopes from the old context are restored immediately
                         ops.push((index, ScopeStackOp::Restore));
                     }
@@ -563,27 +613,29 @@ impl ParseState {
                         }
                     }
                 } else {
-                    let mut repush = is_set && (!cur_context.meta_scope.is_empty() || !cur_context.meta_content_scope.is_empty());
-                    if !repush {
-                        for r in context_refs.iter() {
-                            let ctx = r.resolve(syntax_set)?;
-                            if !ctx.meta_content_scope.is_empty() || (ctx.clear_scopes.is_some() && is_set) {
-                                repush = true;
-                                break;
-                            }
-                        }
-                    }
+                    let repush = (is_set
+                        && (!cur_context.meta_scope.is_empty()
+                            || !cur_context.meta_content_scope.is_empty()))
+                        || context_refs.iter().any(|r| {
+                            let ctx = r.resolve(syntax_set).unwrap();
+
+                            !ctx.meta_content_scope.is_empty()
+                                || (ctx.clear_scopes.is_some() && is_set)
+                        });
                     if repush {
                         // remove previously pushed meta scopes, so that meta content scopes will be applied in the correct order
-                        let mut num_to_pop : usize = 0;
-                        for r in context_refs.iter() {
-                            let ctx = r.resolve(syntax_set)?;
-                            num_to_pop += ctx.meta_scope.len();
-                        }
+                        let mut num_to_pop: usize = context_refs
+                            .iter()
+                            .map(|r| {
+                                let ctx = r.resolve(syntax_set).unwrap();
+                                ctx.meta_scope.len()
+                            })
+                            .sum();
 
                         // also pop off the original context's meta scopes
                         if is_set {
-                            num_to_pop += cur_context.meta_content_scope.len() + cur_context.meta_scope.len();
+                            num_to_pop +=
+                                cur_context.meta_content_scope.len() + cur_context.meta_scope.len();
                         }
 
                         // do all the popping as one operation
@@ -611,9 +663,10 @@ impl ParseState {
                         }
                     }
                 }
-            },
+            }
             MatchOperation::None => (),
         }
+
         Ok(())
     }
 
@@ -623,8 +676,8 @@ impl ParseState {
         line: &str,
         regions: &Region,
         pat: &MatchPattern,
-        syntax_set: &SyntaxSet
-    ) -> bool {
+        syntax_set: &SyntaxSet,
+    ) -> Result<bool, ParsingError> {
         let (ctx_refs, old_proto_ids) = match pat.operation {
             MatchOperation::Push(ref ctx_refs) => (ctx_refs, None),
             MatchOperation::Set(ref ctx_refs) => {
@@ -635,9 +688,9 @@ impl ParseState {
             }
             MatchOperation::Pop => {
                 self.stack.pop();
-                return true;
+                return Ok(true);
             }
-            MatchOperation::None => return false,
+            MatchOperation::None => return Ok(false),
         };
         for (i, r) in ctx_refs.iter().enumerate() {
             let mut proto_ids = if i == 0 {
@@ -654,15 +707,18 @@ impl ParseState {
                 // referred to as the "target" of the push by sublimehq - see
                 // https://forum.sublimetext.com/t/dev-build-3111/19240/17 for more info
                 if let Some(ref p) = pat.with_prototype {
-                    proto_ids.push(p.id());
+                    proto_ids.push(p.id()?);
                 }
             }
-            let context_id = r.id();
-            let context = syntax_set.get_context(&context_id);
+            let context_id = r.id()?;
+            let context = syntax_set.get_context(&context_id)?;
             let captures = {
                 let mut uses_backrefs = context.uses_backrefs;
                 if !proto_ids.is_empty() {
-                    uses_backrefs = uses_backrefs || proto_ids.iter().any(|id| syntax_set.get_context(id).uses_backrefs);
+                    uses_backrefs = uses_backrefs
+                        || proto_ids
+                            .iter()
+                            .any(|id| syntax_set.get_context(id).unwrap().uses_backrefs);
                 }
                 if uses_backrefs {
                     Some((regions.clone(), line.to_owned()))
@@ -676,7 +732,7 @@ impl ParseState {
                 captures,
             });
         }
-        true
+        Ok(true)
     }
 }
 
@@ -684,8 +740,8 @@ impl ParseState {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::parsing::{SyntaxSet, SyntaxSetBuilder, Scope, ScopeStack};
-    use crate::parsing::ScopeStackOp::{Push, Pop, Clear, Restore};
+    use crate::parsing::ScopeStackOp::{Clear, Pop, Push, Restore};
+    use crate::parsing::{Scope, ScopeStack, SyntaxSet, SyntaxSetBuilder};
     use crate::util::debug_print_ops;
 
     const TEST_SYNTAX: &str = include_str!("../../testdata/parser_tests.sublime-syntax");
@@ -721,7 +777,7 @@ mod tests {
             (3, Pop(2)),
             (3, Push(Scope::new("meta.function.ruby").unwrap())),
             (4, Push(Scope::new("entity.name.function.ruby").unwrap())),
-            (7, Pop(1))
+            (7, Pop(1)),
         ];
         assert_eq!(&ops2[0..test_ops2.len()], &test_ops2[..]);
     }
@@ -734,16 +790,28 @@ mod tests {
             ParseState::new(syntax)
         };
 
-        assert_eq!(ops(&mut state, "key: value\n", &ps), vec![
-            (0, Push(Scope::new("source.yaml").unwrap())),
-            (0, Push(Scope::new("string.unquoted.plain.out.yaml").unwrap())),
-            (0, Push(Scope::new("entity.name.tag.yaml").unwrap())),
-            (3, Pop(2)),
-            (3, Push(Scope::new("punctuation.separator.key-value.mapping.yaml").unwrap())),
-            (4, Pop(1)),
-            (5, Push(Scope::new("string.unquoted.plain.out.yaml").unwrap())),
-            (10, Pop(1)),
-        ]);
+        assert_eq!(
+            ops(&mut state, "key: value\n", &ps),
+            vec![
+                (0, Push(Scope::new("source.yaml").unwrap())),
+                (
+                    0,
+                    Push(Scope::new("string.unquoted.plain.out.yaml").unwrap())
+                ),
+                (0, Push(Scope::new("entity.name.tag.yaml").unwrap())),
+                (3, Pop(2)),
+                (
+                    3,
+                    Push(Scope::new("punctuation.separator.key-value.mapping.yaml").unwrap())
+                ),
+                (4, Pop(1)),
+                (
+                    5,
+                    Push(Scope::new("string.unquoted.plain.out.yaml").unwrap())
+                ),
+                (10, Pop(1)),
+            ]
+        );
     }
 
     #[test]
@@ -766,8 +834,8 @@ mod tests {
         test_stack.push(Scope::new("meta.function.parameters.ruby").unwrap());
 
         let mut stack = ScopeStack::new();
-        for &(_, ref op) in ops.iter() {
-            stack.apply(op);
+        for (_, op) in ops.iter() {
+            stack.apply(op).expect("#[cfg(test)]");
         }
         assert_eq!(stack, test_stack);
     }
@@ -783,30 +851,51 @@ mod tests {
         // For parsing HEREDOC, the "SQL" is captured at the beginning and then used in another
         // regex with a backref, to match the end of the HEREDOC. Note that there can be code
         // after the marker (`.strip`) here.
-        assert_eq!(ops(&mut state, "lol = <<-SQL.strip", &ss), vec![
-            (0, Push(Scope::new("source.ruby.rails").unwrap())),
-            (4, Push(Scope::new("keyword.operator.assignment.ruby").unwrap())),
-            (5, Pop(1)),
-            (6, Push(Scope::new("string.unquoted.embedded.sql.ruby").unwrap())),
-            (6, Push(Scope::new("punctuation.definition.string.begin.ruby").unwrap())),
-            (12, Pop(1)),
-            (12, Pop(1)),
-            (12, Push(Scope::new("string.unquoted.embedded.sql.ruby").unwrap())),
-            (12, Push(Scope::new("text.sql.embedded.ruby").unwrap())),
-            (12, Clear(ClearAmount::TopN(2))),
-            (12, Push(Scope::new("punctuation.accessor.ruby").unwrap())),
-            (13, Pop(1)),
-            (18, Restore),
-        ]);
+        assert_eq!(
+            ops(&mut state, "lol = <<-SQL.strip", &ss),
+            vec![
+                (0, Push(Scope::new("source.ruby.rails").unwrap())),
+                (
+                    4,
+                    Push(Scope::new("keyword.operator.assignment.ruby").unwrap())
+                ),
+                (5, Pop(1)),
+                (
+                    6,
+                    Push(Scope::new("string.unquoted.embedded.sql.ruby").unwrap())
+                ),
+                (
+                    6,
+                    Push(Scope::new("punctuation.definition.string.begin.ruby").unwrap())
+                ),
+                (12, Pop(1)),
+                (12, Pop(1)),
+                (
+                    12,
+                    Push(Scope::new("string.unquoted.embedded.sql.ruby").unwrap())
+                ),
+                (12, Push(Scope::new("text.sql.embedded.ruby").unwrap())),
+                (12, Clear(ClearAmount::TopN(2))),
+                (12, Push(Scope::new("punctuation.accessor.ruby").unwrap())),
+                (13, Pop(1)),
+                (18, Restore),
+            ]
+        );
 
         assert_eq!(ops(&mut state, "wow", &ss), vec![]);
 
-        assert_eq!(ops(&mut state, "SQL", &ss), vec![
-            (0, Pop(1)),
-            (0, Push(Scope::new("punctuation.definition.string.end.ruby").unwrap())),
-            (3, Pop(1)),
-            (3, Pop(1)),
-        ]);
+        assert_eq!(
+            ops(&mut state, "SQL", &ss),
+            vec![
+                (0, Pop(1)),
+                (
+                    0,
+                    Push(Scope::new("punctuation.definition.string.end.ruby").unwrap())
+                ),
+                (3, Pop(1)),
+                (3, Pop(1)),
+            ]
+        );
     }
 
     #[test]
@@ -817,47 +906,77 @@ mod tests {
             ParseState::new(syntax)
         };
 
-        assert_eq!(ops(&mut state, "#ifdef FOO", &ss), vec![
-            (0, Push(Scope::new("source.c").unwrap())),
-            (0, Push(Scope::new("meta.preprocessor.c").unwrap())),
-            (0, Push(Scope::new("keyword.control.import.c").unwrap())),
-            (6, Pop(1)),
-            (10, Pop(1)),
-        ]);
-        assert_eq!(ops(&mut state, "{", &ss), vec![
-            (0, Push(Scope::new("meta.block.c").unwrap())),
-            (0, Push(Scope::new("punctuation.section.block.begin.c").unwrap())),
-            (1, Pop(1)),
-        ]);
-        assert_eq!(ops(&mut state, "#else", &ss), vec![
-            (0, Push(Scope::new("meta.preprocessor.c").unwrap())),
-            (0, Push(Scope::new("keyword.control.import.c").unwrap())),
-            (5, Pop(1)),
-            (5, Pop(1)),
-        ]);
-        assert_eq!(ops(&mut state, "{", &ss), vec![
-            (0, Push(Scope::new("meta.block.c").unwrap())),
-            (0, Push(Scope::new("punctuation.section.block.begin.c").unwrap())),
-            (1, Pop(1)),
-        ]);
-        assert_eq!(ops(&mut state, "#endif", &ss), vec![
-            (0, Pop(1)),
-            (0, Push(Scope::new("meta.block.c").unwrap())),
-            (0, Push(Scope::new("meta.preprocessor.c").unwrap())),
-            (0, Push(Scope::new("keyword.control.import.c").unwrap())),
-            (6, Pop(2)),
-            (6, Pop(2)),
-            (6, Push(Scope::new("meta.block.c").unwrap())),
-        ]);
-        assert_eq!(ops(&mut state, "    foo;", &ss), vec![
-            (7, Push(Scope::new("punctuation.terminator.c").unwrap())),
-            (8, Pop(1)),
-        ]);
-        assert_eq!(ops(&mut state, "}", &ss), vec![
-            (0, Push(Scope::new("punctuation.section.block.end.c").unwrap())),
-            (1, Pop(1)),
-            (1, Pop(1)),
-        ]);
+        assert_eq!(
+            ops(&mut state, "#ifdef FOO", &ss),
+            vec![
+                (0, Push(Scope::new("source.c").unwrap())),
+                (0, Push(Scope::new("meta.preprocessor.c").unwrap())),
+                (0, Push(Scope::new("keyword.control.import.c").unwrap())),
+                (6, Pop(1)),
+                (10, Pop(1)),
+            ]
+        );
+        assert_eq!(
+            ops(&mut state, "{", &ss),
+            vec![
+                (0, Push(Scope::new("meta.block.c").unwrap())),
+                (
+                    0,
+                    Push(Scope::new("punctuation.section.block.begin.c").unwrap())
+                ),
+                (1, Pop(1)),
+            ]
+        );
+        assert_eq!(
+            ops(&mut state, "#else", &ss),
+            vec![
+                (0, Push(Scope::new("meta.preprocessor.c").unwrap())),
+                (0, Push(Scope::new("keyword.control.import.c").unwrap())),
+                (5, Pop(1)),
+                (5, Pop(1)),
+            ]
+        );
+        assert_eq!(
+            ops(&mut state, "{", &ss),
+            vec![
+                (0, Push(Scope::new("meta.block.c").unwrap())),
+                (
+                    0,
+                    Push(Scope::new("punctuation.section.block.begin.c").unwrap())
+                ),
+                (1, Pop(1)),
+            ]
+        );
+        assert_eq!(
+            ops(&mut state, "#endif", &ss),
+            vec![
+                (0, Pop(1)),
+                (0, Push(Scope::new("meta.block.c").unwrap())),
+                (0, Push(Scope::new("meta.preprocessor.c").unwrap())),
+                (0, Push(Scope::new("keyword.control.import.c").unwrap())),
+                (6, Pop(2)),
+                (6, Pop(2)),
+                (6, Push(Scope::new("meta.block.c").unwrap())),
+            ]
+        );
+        assert_eq!(
+            ops(&mut state, "    foo;", &ss),
+            vec![
+                (7, Push(Scope::new("punctuation.terminator.c").unwrap())),
+                (8, Pop(1)),
+            ]
+        );
+        assert_eq!(
+            ops(&mut state, "}", &ss),
+            vec![
+                (
+                    0,
+                    Push(Scope::new("punctuation.section.block.end.c").unwrap())
+                ),
+                (1, Pop(1)),
+                (1, Pop(1)),
+            ]
+        );
     }
 
     #[test]
@@ -924,9 +1043,7 @@ mod tests {
     #[test]
     fn can_parse_infinite_loop() {
         let line = "#infinite_loop_test 123";
-        let expect = [
-            "<source.test>, <constant.numeric.test>",
-        ];
+        let expect = ["<source.test>, <constant.numeric.test>"];
         expect_scope_stacks(line, &expect, TEST_SYNTAX);
     }
 
@@ -995,8 +1112,9 @@ contexts:
         let syntax = SyntaxDefinition::load_from_str(
             include_str!("../../testdata/embed_escape_test.sublime-syntax"),
             false,
-            None
-        ).unwrap();
+            None,
+        )
+        .unwrap();
 
         let line1 = "\"abctest\" foobar";
         let expect1 = [
@@ -1357,11 +1475,15 @@ contexts:
 "#;
 
         let stack_states = stack_states(parse("aa b", syntax));
-        assert_eq!(stack_states, vec![
-            "<source.test>",
-            "<source.test>, <test.good>",
-            "<source.test>",
-        ], "Expected test.bad to not match");
+        assert_eq!(
+            stack_states,
+            vec![
+                "<source.test>",
+                "<source.test>, <test.good>",
+                "<source.test>",
+            ],
+            "Expected test.bad to not match"
+        );
     }
 
     #[test]
@@ -1415,14 +1537,23 @@ contexts:
         expect_scope_stacks_with_syntax(
             "a1b2c3d4e5",
             &[
-                "<a>", "<1>", "<b>", "<2>", "<c>", "<3>", "<d>", "<4>", "<e>", "<5>"
-            ], SyntaxDefinition::load_from_str(syntax, true, None).unwrap()
+                "<a>", "<1>", "<b>", "<2>", "<c>", "<3>", "<d>", "<4>", "<e>", "<5>",
+            ],
+            SyntaxDefinition::load_from_str(syntax, true, None).unwrap(),
         );
         expect_scope_stacks_with_syntax(
             "5cfcecbedcdea",
             &[
-                "<5>", "<cwith>", "<f>", "<e>", "<b>", "<d>", "<cwithout>", "<a>"
-            ], SyntaxDefinition::load_from_str(syntax, true, None).unwrap()
+                "<5>",
+                "<cwith>",
+                "<f>",
+                "<e>",
+                "<b>",
+                "<d>",
+                "<cwithout>",
+                "<a>",
+            ],
+            SyntaxDefinition::load_from_str(syntax, true, None).unwrap(),
         );
     }
 
@@ -1444,7 +1575,11 @@ contexts:
 "#;
 
         let syntax = SyntaxDefinition::load_from_str(syntax, true, None).unwrap();
-        expect_scope_stacks_with_syntax("testfoo", &["<test>", /*"<ignored>",*/ "<f>", "<keyword>"], syntax);
+        expect_scope_stacks_with_syntax(
+            "testfoo",
+            &["<test>", /*"<ignored>",*/ "<f>", "<keyword>"],
+            syntax,
+        );
     }
 
     #[test]
@@ -1649,27 +1784,30 @@ contexts:
         let syntax_set = link(syntax_newlines);
 
         let mut state = ParseState::new(&syntax_set.syntaxes()[0]);
-        assert_eq!(ops(&mut state, "foo\n", &syntax_set), vec![
-            (0, Push(Scope::new("source.test").unwrap())),
-            (0, Push(Scope::new("word").unwrap())),
-            (3, Pop(1))
-        ]);
-        assert_eq!(ops(&mut state, "===\n", &syntax_set), vec![
-            (0, Push(Scope::new("heading").unwrap())),
-            (3, Pop(1))
-        ]);
+        assert_eq!(
+            ops(&mut state, "foo\n", &syntax_set),
+            vec![
+                (0, Push(Scope::new("source.test").unwrap())),
+                (0, Push(Scope::new("word").unwrap())),
+                (3, Pop(1))
+            ]
+        );
+        assert_eq!(
+            ops(&mut state, "===\n", &syntax_set),
+            vec![(0, Push(Scope::new("heading").unwrap())), (3, Pop(1))]
+        );
 
-        assert_eq!(ops(&mut state, "bar\n", &syntax_set), vec![
-            (0, Push(Scope::new("word").unwrap())),
-            (3, Pop(1))
-        ]);
+        assert_eq!(
+            ops(&mut state, "bar\n", &syntax_set),
+            vec![(0, Push(Scope::new("word").unwrap())), (3, Pop(1))]
+        );
         // This should result in popping out of the context
         assert_eq!(ops(&mut state, "\n", &syntax_set), vec![]);
         // So now this matches other
-        assert_eq!(ops(&mut state, "====\n", &syntax_set), vec![
-            (0, Push(Scope::new("other").unwrap())),
-            (4, Pop(1))
-        ]);
+        assert_eq!(
+            ops(&mut state, "====\n", &syntax_set),
+            vec![(0, Push(Scope::new("other").unwrap())), (4, Pop(1))]
+        );
     }
 
     #[test]
@@ -1687,14 +1825,17 @@ contexts:
         let syntax_set = link(syntax_newlines);
 
         let mut state = ParseState::new(&syntax_set.syntaxes()[0]);
-        assert_eq!(ops(&mut state, "// foo\n", &syntax_set), vec![
-            (0, Push(Scope::new("source.test").unwrap())),
-            (0, Push(Scope::new("comment.line.double-slash").unwrap())),
-            // 6 is important here, should not be 7. The pattern should *not* consume the newline,
-            // but instead match before it. This is important for whitespace-sensitive syntaxes
-            // where newlines terminate statements such as Scala.
-            (6, Pop(1))
-        ]);
+        assert_eq!(
+            ops(&mut state, "// foo\n", &syntax_set),
+            vec![
+                (0, Push(Scope::new("source.test").unwrap())),
+                (0, Push(Scope::new("comment.line.double-slash").unwrap())),
+                // 6 is important here, should not be 7. The pattern should *not* consume the newline,
+                // but instead match before it. This is important for whitespace-sensitive syntaxes
+                // where newlines terminate statements such as Scala.
+                (6, Pop(1))
+            ]
+        );
     }
 
     #[test]
@@ -1723,7 +1864,8 @@ contexts:
 
     #[test]
     fn can_include_backrefs() {
-        let syntax = SyntaxDefinition::load_from_str(r#"
+        let syntax = SyntaxDefinition::load_from_str(
+            r#"
                 name: Backref Include Test
                 scope: source.backrefinc
                 contexts:
@@ -1737,14 +1879,19 @@ contexts:
                     - match: \1
                       scope: b
                       pop: true
-                "#, true, None).unwrap();
+                "#,
+            true,
+            None,
+        )
+        .unwrap();
 
         expect_scope_stacks_with_syntax("aa", &["<a>", "<b>"], syntax);
     }
 
     #[test]
     fn can_include_nested_backrefs() {
-        let syntax = SyntaxDefinition::load_from_str(r#"
+        let syntax = SyntaxDefinition::load_from_str(
+            r#"
                 name: Backref Include Test
                 scope: source.backrefinc
                 contexts:
@@ -1760,7 +1907,11 @@ contexts:
                     - match: \1
                       scope: b
                       pop: true
-                "#, true, None).unwrap();
+                "#,
+            true,
+            None,
+        )
+        .unwrap();
 
         expect_scope_stacks_with_syntax("aa", &["<a>", "<b>"], syntax);
     }
@@ -1814,8 +1965,12 @@ contexts:
         builder.build()
     }
 
-    fn ops(state: &mut ParseState, line: &str, syntax_set: &SyntaxSet) -> Vec<(usize, ScopeStackOp)> {
-        let ops = state.parse_line(line, syntax_set).unwrap();
+    fn ops(
+        state: &mut ParseState,
+        line: &str,
+        syntax_set: &SyntaxSet,
+    ) -> Vec<(usize, ScopeStackOp)> {
+        let ops = state.parse_line(line, syntax_set).expect("#[cfg(test)]");
         debug_print_ops(line, &ops);
         ops
     }
@@ -1823,9 +1978,13 @@ contexts:
     fn stack_states(ops: Vec<(usize, ScopeStackOp)>) -> Vec<String> {
         let mut states = Vec::new();
         let mut stack = ScopeStack::new();
-        for &(_, ref op) in ops.iter() {
-            stack.apply(op);
-            let scopes: Vec<String> = stack.as_slice().iter().map(|s| format!("{:?}", s)).collect();
+        for (_, op) in ops.iter() {
+            stack.apply(op).expect("#[cfg(test)]");
+            let scopes: Vec<String> = stack
+                .as_slice()
+                .iter()
+                .map(|s| format!("{:?}", s))
+                .collect();
             let stack_str = scopes.join(", ");
             states.push(stack_str);
         }
