@@ -1,71 +1,49 @@
 use super::regex::{Regex, Region};
 use super::scope::*;
 use super::syntax_definition::*;
-use yaml_rust::{YamlLoader, Yaml, ScanError};
-use yaml_rust::yaml::Hash;
 use std::collections::HashMap;
 use std::error::Error;
-use std::fmt;
-use std::path::Path;
 use std::ops::DerefMut;
+use std::path::Path;
+use yaml_rust::yaml::Hash;
+use yaml_rust::{ScanError, Yaml, YamlLoader};
 
-#[derive(Debug)]
+#[derive(Debug, thiserror::Error)]
+#[non_exhaustive]
 pub enum ParseSyntaxError {
     /// Invalid YAML file syntax, or at least something yaml_rust can't handle
-    InvalidYaml(ScanError),
+    #[error("Invalid YAML file syntax: {0}")]
+    InvalidYaml(#[from] ScanError),
     /// The file must contain at least one YAML document
+    #[error("The file must contain at least one YAML document")]
     EmptyFile,
     /// Some keys are required for something to be a valid `.sublime-syntax`
+    #[error("Missing mandatory key in YAML file: {0}")]
     MissingMandatoryKey(&'static str),
     /// Invalid regex
-    RegexCompileError(String, Box<dyn Error + Send + Sync + 'static>),
+    #[error("Error while compiling regex '{0}': {1}")]
+    RegexCompileError(String, #[source] Box<dyn Error + Send + Sync + 'static>),
     /// A scope that syntect's scope implementation can't handle
+    #[error("Invalid scope: {0}")]
     InvalidScope(ParseScopeError),
     /// A reference to another file that is invalid
+    #[error("Invalid file reference")]
     BadFileRef,
     /// Syntaxes must have a context named "main"
+    #[error("Context 'main' is missing")]
     MainMissing,
     /// Some part of the YAML file is the wrong type (e.g a string but should be a list)
     /// Sorry this doesn't give you any way to narrow down where this is.
     /// Maybe use Sublime Text to figure it out.
+    #[error("Type mismatch")]
     TypeMismatch,
 }
 
-impl fmt::Display for ParseSyntaxError {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        use crate::ParseSyntaxError::*;
-
-        match *self {
-            InvalidYaml(ref err) => write!(f, "Invalid YAML file syntax: {}", err),
-            EmptyFile => write!(f, "Empty file"),
-            MissingMandatoryKey(key) => write!(f, "Missing mandatory key in YAML file: {}", key),
-            RegexCompileError(ref regex, ref error) => {
-                write!(f, "Error while compiling regex '{}': {}", regex, error)
-            }
-            InvalidScope(_) => write!(f, "Invalid scope"),
-            BadFileRef => write!(f, "Invalid file reference"),
-            MainMissing => write!(f, "Context 'main' is missing"),
-            TypeMismatch => write!(f, "Type mismatch"),
-        }
-    }
-}
-
-impl Error for ParseSyntaxError {
-    fn cause(&self) -> Option<&dyn Error> {
-        use crate::ParseSyntaxError::*;
-
-        match self {
-            InvalidYaml(ref error) => Some(error),
-            RegexCompileError(_, error) => Some(error.as_ref()),
-            _ => None,
-        }
-    }
-}
-
-fn get_key<'a, R, F: FnOnce(&'a Yaml) -> Option<R>>(map: &'a Hash,
-                                                    key: &'static str,
-                                                    f: F)
-                                                    -> Result<R, ParseSyntaxError> {
+fn get_key<'a, R, F: FnOnce(&'a Yaml) -> Option<R>>(
+    map: &'a Hash,
+    key: &'static str,
+    f: F,
+) -> Result<R, ParseSyntaxError> {
     map.get(&Yaml::String(key.to_owned()))
         .ok_or(ParseSyntaxError::MissingMandatoryKey(key))
         .and_then(|x| f(x).ok_or(ParseSyntaxError::TypeMismatch))
@@ -118,14 +96,20 @@ impl SyntaxDefinition {
         }
         let doc = &docs[0];
         let mut scope_repo = SCOPE_REPO.lock().unwrap();
-        SyntaxDefinition::parse_top_level(doc, scope_repo.deref_mut(), lines_include_newline, fallback_name)
+        SyntaxDefinition::parse_top_level(
+            doc,
+            scope_repo.deref_mut(),
+            lines_include_newline,
+            fallback_name,
+        )
     }
 
-    fn parse_top_level(doc: &Yaml,
-                       scope_repo: &mut ScopeRepository,
-                       lines_include_newline: bool,
-                       fallback_name: Option<&str>)
-                       -> Result<SyntaxDefinition, ParseSyntaxError> {
+    fn parse_top_level(
+        doc: &Yaml,
+        scope_repo: &mut ScopeRepository,
+        lines_include_newline: bool,
+        fallback_name: Option<&str>,
+    ) -> Result<SyntaxDefinition, ParseSyntaxError> {
         let h = doc.as_hash().ok_or(ParseSyntaxError::TypeMismatch)?;
 
         let mut variables = HashMap::new();
@@ -137,7 +121,8 @@ impl SyntaxDefinition {
             }
         }
         let contexts_hash = get_key(h, "contexts", |x| x.as_hash())?;
-        let top_level_scope = scope_repo.build(get_key(h, "scope", |x| x.as_str())?)
+        let top_level_scope = scope_repo
+            .build(get_key(h, "scope", |x| x.as_str())?)
             .map_err(ParseSyntaxError::InvalidScope)?;
         let mut state = ParserState {
             scope_repo,
@@ -152,20 +137,21 @@ impl SyntaxDefinition {
             return Err(ParseSyntaxError::MainMissing);
         }
 
-        SyntaxDefinition::add_initial_contexts(
-            &mut contexts,
-            &mut state,
-            top_level_scope,
-        );
+        SyntaxDefinition::add_initial_contexts(&mut contexts, &mut state, top_level_scope);
+
+        let mut file_extensions = Vec::new();
+        for extension_key in &["file_extensions", "hidden_file_extensions"] {
+            if let Ok(v) = get_key(h, extension_key, |x| x.as_vec()) {
+                file_extensions.extend(v.iter().filter_map(|y| y.as_str().map(|s| s.to_owned())))
+            }
+        }
 
         let defn = SyntaxDefinition {
-            name: get_key(h, "name", |x| x.as_str()).unwrap_or_else(|_| fallback_name.unwrap_or("Unnamed")).to_owned(),
+            name: get_key(h, "name", |x| x.as_str())
+                .unwrap_or_else(|_| fallback_name.unwrap_or("Unnamed"))
+                .to_owned(),
             scope: top_level_scope,
-            file_extensions: {
-                get_key(h, "file_extensions", |x| x.as_vec())
-                    .map(|v| v.iter().filter_map(|y| y.as_str()).map(|x| x.to_owned()).collect())
-                    .unwrap_or_else(|_| Vec::new())
-            },
+            file_extensions,
             // TODO maybe cache a compiled version of this Regex
             first_line_match: get_key(h, "first_line_match", |x| x.as_str())
                 .ok()
@@ -178,28 +164,36 @@ impl SyntaxDefinition {
         Ok(defn)
     }
 
-    fn parse_contexts(map: &Hash,
-                      state: &mut ParserState<'_>)
-                      -> Result<HashMap<String, Context>, ParseSyntaxError> {
+    fn parse_contexts(
+        map: &Hash,
+        state: &mut ParserState<'_>,
+    ) -> Result<HashMap<String, Context>, ParseSyntaxError> {
         let mut contexts = HashMap::new();
         for (key, value) in map.iter() {
             if let (Some(name), Some(val_vec)) = (key.as_str(), value.as_vec()) {
                 let is_prototype = name == "prototype";
                 let mut namer = ContextNamer::new(name);
-                SyntaxDefinition::parse_context(val_vec, state, &mut contexts, is_prototype, &mut namer)?;
+                SyntaxDefinition::parse_context(
+                    val_vec,
+                    state,
+                    &mut contexts,
+                    is_prototype,
+                    &mut namer,
+                )?;
             }
         }
 
         Ok(contexts)
     }
 
-    fn parse_context(vec: &[Yaml],
-                     // TODO: Maybe just pass the scope repo if that's all that's needed?
-                     state: &mut ParserState<'_>,
-                     contexts: &mut HashMap<String, Context>,
-                     is_prototype: bool,
-                     namer: &mut ContextNamer)
-                     -> Result<String, ParseSyntaxError> {
+    fn parse_context(
+        vec: &[Yaml],
+        // TODO: Maybe just pass the scope repo if that's all that's needed?
+        state: &mut ParserState<'_>,
+        contexts: &mut HashMap<String, Context>,
+        is_prototype: bool,
+        namer: &mut ContextNamer,
+    ) -> Result<String, ParseSyntaxError> {
         let mut context = Context::new(!is_prototype);
         let name = namer.next();
 
@@ -229,30 +223,31 @@ impl SyntaxDefinition {
             }
             if !is_special {
                 if let Ok(x) = get_key(map, "include", Some) {
-                    let reference = SyntaxDefinition::parse_reference(
-                        x, state, contexts, namer)?;
+                    let reference =
+                        SyntaxDefinition::parse_reference(x, state, contexts, namer, false)?;
                     context.patterns.push(Pattern::Include(reference));
                 } else {
-                    let pattern = SyntaxDefinition::parse_match_pattern(
-                        map, state, contexts, namer)?;
+                    let pattern =
+                        SyntaxDefinition::parse_match_pattern(map, state, contexts, namer)?;
                     if pattern.has_captures {
                         context.uses_backrefs = true;
                     }
                     context.patterns.push(Pattern::Match(pattern));
                 }
             }
-
         }
 
         contexts.insert(name.clone(), context);
         Ok(name)
     }
 
-    fn parse_reference(y: &Yaml,
-                       state: &mut ParserState<'_>,
-                       contexts: &mut HashMap<String, Context>,
-                       namer: &mut ContextNamer)
-                       -> Result<ContextReference, ParseSyntaxError> {
+    fn parse_reference(
+        y: &Yaml,
+        state: &mut ParserState<'_>,
+        contexts: &mut HashMap<String, Context>,
+        namer: &mut ContextNamer,
+        with_escape: bool,
+    ) -> Result<ContextReference, ParseSyntaxError> {
         if let Some(s) = y.as_str() {
             let parts: Vec<&str> = s.split('#').collect();
             let sub_context = if parts.len() > 1 {
@@ -262,10 +257,12 @@ impl SyntaxDefinition {
             };
             if parts[0].starts_with("scope:") {
                 Ok(ContextReference::ByScope {
-                    scope: state.scope_repo
+                    scope: state
+                        .scope_repo
                         .build(&parts[0][6..])
                         .map_err(ParseSyntaxError::InvalidScope)?,
                     sub_context,
+                    with_escape,
                 })
             } else if parts[0].ends_with(".sublime-syntax") {
                 let stem = Path::new(parts[0])
@@ -275,6 +272,7 @@ impl SyntaxDefinition {
                 Ok(ContextReference::File {
                     name: stem.to_owned(),
                     sub_context,
+                    with_escape,
                 })
             } else {
                 Ok(ContextReference::Named(parts[0].to_owned()))
@@ -287,11 +285,12 @@ impl SyntaxDefinition {
         }
     }
 
-    fn parse_match_pattern(map: &Hash,
-                           state: &mut ParserState<'_>,
-                           contexts: &mut HashMap<String, Context>,
-                           namer: &mut ContextNamer)
-                           -> Result<MatchPattern, ParseSyntaxError> {
+    fn parse_match_pattern(
+        map: &Hash,
+        state: &mut ParserState<'_>,
+        contexts: &mut HashMap<String, Context>,
+        namer: &mut ContextNamer,
+    ) -> Result<MatchPattern, ParseSyntaxError> {
         let raw_regex = get_key(map, "match", |x| x.as_str())?;
         let regex_str = Self::parse_regex(raw_regex, state)?;
         // println!("{:?}", regex_str);
@@ -310,7 +309,9 @@ impl SyntaxDefinition {
         let mut has_captures = false;
         let operation = if get_key(map, "pop", Some).is_ok() {
             // Thanks @wbond for letting me know this is the correct way to check for captures
-            has_captures = state.backref_regex.search(&regex_str, 0, regex_str.len(), None);
+            has_captures = state
+                .backref_regex
+                .search(&regex_str, 0, regex_str.len(), None);
             MatchOperation::Pop
         } else if let Ok(y) = get_key(map, "push", Some) {
             MatchOperation::Push(SyntaxDefinition::parse_pushargs(y, state, contexts, namer)?)
@@ -318,9 +319,12 @@ impl SyntaxDefinition {
             MatchOperation::Set(SyntaxDefinition::parse_pushargs(y, state, contexts, namer)?)
         } else if let Ok(y) = get_key(map, "embed", Some) {
             // Same as push so we translate it to what it would be
-            let mut embed_escape_context_yaml = vec!();
+            let mut embed_escape_context_yaml = vec![];
             let mut commands = Hash::new();
-            commands.insert(Yaml::String("meta_include_prototype".to_string()), Yaml::Boolean(false));
+            commands.insert(
+                Yaml::String("meta_include_prototype".to_string()),
+                Yaml::Boolean(false),
+            );
             embed_escape_context_yaml.push(Yaml::Hash(commands));
             if let Ok(s) = get_key(map, "embed_scope", Some) {
                 commands = Hash::new();
@@ -342,12 +346,13 @@ impl SyntaxDefinition {
                     false,
                     namer,
                 )?;
-                MatchOperation::Push(vec![ContextReference::Inline(escape_context),
-                                          SyntaxDefinition::parse_reference(y, state, contexts, namer)?])
+                MatchOperation::Push(vec![
+                    ContextReference::Inline(escape_context),
+                    SyntaxDefinition::parse_reference(y, state, contexts, namer, true)?,
+                ])
             } else {
                 return Err(ParseSyntaxError::MissingMandatoryKey("escape"));
             }
-
         } else {
             MatchOperation::None
         };
@@ -361,9 +366,13 @@ impl SyntaxDefinition {
 
             let mut context = Context::new(false);
             let mut match_map = Hash::new();
-            match_map.insert(Yaml::String("match".to_string()), Yaml::String(format!("(?={})", v.as_str().unwrap())));
+            match_map.insert(
+                Yaml::String("match".to_string()),
+                Yaml::String(format!("(?={})", v.as_str().unwrap())),
+            );
             match_map.insert(Yaml::String("pop".to_string()), Yaml::Boolean(true));
-            let pattern = SyntaxDefinition::parse_match_pattern(&match_map, state, contexts, namer)?;
+            let pattern =
+                SyntaxDefinition::parse_match_pattern(&match_map, state, contexts, namer)?;
             if pattern.has_captures {
                 context.uses_backrefs = true;
             }
@@ -387,21 +396,26 @@ impl SyntaxDefinition {
         Ok(pattern)
     }
 
-    fn parse_pushargs(y: &Yaml,
-                      state: &mut ParserState<'_>,
-                      contexts: &mut HashMap<String, Context>,
-                      namer: &mut ContextNamer)
-                      -> Result<Vec<ContextReference>, ParseSyntaxError> {
+    fn parse_pushargs(
+        y: &Yaml,
+        state: &mut ParserState<'_>,
+        contexts: &mut HashMap<String, Context>,
+        namer: &mut ContextNamer,
+    ) -> Result<Vec<ContextReference>, ParseSyntaxError> {
         // check for a push of multiple items
-        if y.as_vec().map_or(false, |v| !v.is_empty() && (v[0].as_str().is_some() || (v[0].as_vec().is_some() && v[0].as_vec().unwrap()[0].as_hash().is_some()))) {
+        if y.as_vec().map_or(false, |v| {
+            !v.is_empty()
+                && (v[0].as_str().is_some()
+                    || (v[0].as_vec().is_some() && v[0].as_vec().unwrap()[0].as_hash().is_some()))
+        }) {
             // this works because Result implements FromIterator to handle the errors
             y.as_vec()
                 .unwrap()
                 .iter()
-                .map(|x| SyntaxDefinition::parse_reference(x, state, contexts, namer))
+                .map(|x| SyntaxDefinition::parse_reference(x, state, contexts, namer, false))
                 .collect()
         } else {
-            let reference = SyntaxDefinition::parse_reference(y, state, contexts, namer)?;
+            let reference = SyntaxDefinition::parse_reference(y, state, contexts, namer, false)?;
             Ok(vec![reference])
         }
     }
@@ -425,14 +439,21 @@ impl SyntaxDefinition {
         let mut result = String::new();
         let mut index = 0;
         let mut region = Region::new();
-        while state.variable_regex.search(raw_regex, index, raw_regex.len(), Some(&mut region)) {
+        while state
+            .variable_regex
+            .search(raw_regex, index, raw_regex.len(), Some(&mut region))
+        {
             let (begin, end) = region.pos(0).unwrap();
 
             result.push_str(&raw_regex[index..begin]);
 
             let var_pos = region.pos(1).unwrap();
             let var_name = &raw_regex[var_pos.0..var_pos.1];
-            let var_raw = state.variables.get(var_name).map(String::as_ref).unwrap_or("");
+            let var_raw = state
+                .variables
+                .get(var_name)
+                .map(String::as_ref)
+                .unwrap_or("");
             let var_resolved = Self::resolve_variables(var_raw, state);
             result.push_str(&var_resolved);
 
@@ -446,7 +467,8 @@ impl SyntaxDefinition {
 
     fn try_compile_regex(regex_str: &str) -> Result<(), ParseSyntaxError> {
         // Replace backreferences with a placeholder value that will also appear in errors
-        let regex_str = substitute_backrefs_in_regex(regex_str, |i| Some(format!("<placeholder_{}>", i)));
+        let regex_str =
+            substitute_backrefs_in_regex(regex_str, |i| Some(format!("<placeholder_{}>", i)));
 
         if let Some(error) = Regex::try_compile(&regex_str) {
             Err(ParseSyntaxError::RegexCompileError(regex_str, error))
@@ -486,14 +508,28 @@ impl SyntaxDefinition {
         let yaml_docs = YamlLoader::load_from_str(START_CONTEXT).unwrap();
         let yaml = &yaml_docs[0];
 
-        let start_yaml : &[Yaml] = yaml["__start"].as_vec().unwrap();
-        SyntaxDefinition::parse_context(start_yaml, state, contexts, false, &mut ContextNamer::new("__start")).unwrap();
+        let start_yaml: &[Yaml] = yaml["__start"].as_vec().unwrap();
+        SyntaxDefinition::parse_context(
+            start_yaml,
+            state,
+            contexts,
+            false,
+            &mut ContextNamer::new("__start"),
+        )
+        .unwrap();
         if let Some(start) = contexts.get_mut("__start") {
             start.meta_content_scope = vec![top_level_scope];
         }
 
-        let main_yaml : &[Yaml] = yaml["__main"].as_vec().unwrap();
-        SyntaxDefinition::parse_context(main_yaml, state, contexts, false, &mut ContextNamer::new("__main")).unwrap();
+        let main_yaml: &[Yaml] = yaml["__main"].as_vec().unwrap();
+        SyntaxDefinition::parse_context(
+            main_yaml,
+            state,
+            contexts,
+            false,
+            &mut ContextNamer::new("__main"),
+        )
+        .unwrap();
 
         let meta_include_prototype = contexts["main"].meta_include_prototype;
         let meta_scope = contexts["main"].meta_scope.clone();
@@ -544,14 +580,14 @@ impl ContextNamer {
 /// Sublime's syntaxes expect them to match Unicode characters as well, so transform them to
 /// corresponding Unicode character classes.
 fn replace_posix_char_classes(regex: String) -> String {
-    regex.replace("[:alpha:]", r"\p{L}")
+    regex
+        .replace("[:alpha:]", r"\p{L}")
         .replace("[:alnum:]", r"\p{L}\p{N}")
         .replace("[:lower:]", r"\p{Ll}")
         .replace("[:upper:]", r"\p{Lu}")
         .replace("[:digit:]", r"\p{Nd}")
 }
 
-
 /// Some of the regexes include `$` and expect it to match end of line,
 /// e.g. *before* the `\n` in `test\n`.
 ///
@@ -780,10 +816,7 @@ struct Parser<'a> {
 
 impl<'a> Parser<'a> {
     fn new(bytes: &[u8]) -> Parser {
-        Parser {
-            bytes,
-            index: 0,
-        }
+        Parser { bytes, index: 0 }
     }
 
     fn peek(&self) -> Option<u8> {
@@ -851,30 +884,31 @@ impl<'a> Parser<'a> {
     }
 }
 
-
 #[cfg(test)]
 mod tests {
-    use crate::parsing::syntax_definition::*;
-    use crate::parsing::Scope;
     use super::*;
+    use crate::parsing::Scope;
 
     #[test]
     fn can_parse() {
-        let defn: SyntaxDefinition =
-            SyntaxDefinition::load_from_str("name: C\nscope: source.c\ncontexts: {main: []}",
-                                            false, None)
-                .unwrap();
+        let defn: SyntaxDefinition = SyntaxDefinition::load_from_str(
+            "name: C\nscope: source.c\ncontexts: {main: []}",
+            false,
+            None,
+        )
+        .unwrap();
         assert_eq!(defn.name, "C");
         assert_eq!(defn.scope, Scope::new("source.c").unwrap());
         let exts_empty: Vec<String> = Vec::new();
         assert_eq!(defn.file_extensions, exts_empty);
-        assert_eq!(defn.hidden, false);
+        assert!(!defn.hidden);
         assert!(defn.variables.is_empty());
-        let defn2: SyntaxDefinition =
-            SyntaxDefinition::load_from_str("
+        let defn2: SyntaxDefinition = SyntaxDefinition::load_from_str(
+            "
         name: C
         scope: source.c
         file_extensions: [c, h]
+        hidden_file_extensions: [k, l]
         hidden: true
         variables:
           ident: '[QY]+'
@@ -902,14 +936,19 @@ mod tests {
             - match: '\"'
               pop: true
         ",
-                                            false, None)
-                .unwrap();
+            false,
+            None,
+        )
+        .unwrap();
         assert_eq!(defn2.name, "C");
         let top_level_scope = Scope::new("source.c").unwrap();
         assert_eq!(defn2.scope, top_level_scope);
-        let exts: Vec<String> = vec![String::from("c"), String::from("h")];
+        let exts: Vec<String> = vec!["c", "h", "k", "l"]
+            .into_iter()
+            .map(String::from)
+            .collect();
         assert_eq!(defn2.file_extensions, exts);
-        assert_eq!(defn2.hidden, true);
+        assert!(defn2.hidden);
         assert_eq!(defn2.variables.get("ident").unwrap(), "[QY]+");
 
         let n: Vec<Scope> = Vec::new();
@@ -918,35 +957,54 @@ mod tests {
         let main = &defn2.contexts["main"];
         assert_eq!(main.meta_content_scope, vec![top_level_scope]);
         assert_eq!(main.meta_scope, n);
-        assert_eq!(main.meta_include_prototype, true);
+        assert!(main.meta_include_prototype);
 
         assert_eq!(defn2.contexts["__main"].meta_content_scope, n);
-        assert_eq!(defn2.contexts["__start"].meta_content_scope, vec![top_level_scope]);
+        assert_eq!(
+            defn2.contexts["__start"].meta_content_scope,
+            vec![top_level_scope]
+        );
 
-        assert_eq!(defn2.contexts["string"].meta_scope,
-                   vec![Scope::new("string.quoted.double.c").unwrap()]);
+        assert_eq!(
+            defn2.contexts["string"].meta_scope,
+            vec![Scope::new("string.quoted.double.c").unwrap()]
+        );
         let first_pattern: &Pattern = &main.patterns[0];
         match *first_pattern {
             Pattern::Match(ref match_pat) => {
                 let m: &CaptureMapping = match_pat.captures.as_ref().expect("test failed");
-                assert_eq!(&m[0], &(1,vec![Scope::new("meta.preprocessor.c++").unwrap()]));
+                assert_eq!(
+                    &m[0],
+                    &(1, vec![Scope::new("meta.preprocessor.c++").unwrap()])
+                );
                 use crate::parsing::syntax_definition::ContextReference::*;
 
                 // this is sadly necessary because Context is not Eq because of the Regex
                 let expected = MatchOperation::Push(vec![
                     Named("string".to_owned()),
-                    ByScope { scope: Scope::new("source.c").unwrap(), sub_context: Some("main".to_owned()) },
+                    ByScope {
+                        scope: Scope::new("source.c").unwrap(),
+                        sub_context: Some("main".to_owned()),
+                        with_escape: false,
+                    },
                     File {
                         name: "CSS".to_owned(),
-                        sub_context: Some("rule-list-body".to_owned())
+                        sub_context: Some("rule-list-body".to_owned()),
+                        with_escape: false,
                     },
                 ]);
-                assert_eq!(format!("{:?}", match_pat.operation),
-                           format!("{:?}", expected));
+                assert_eq!(
+                    format!("{:?}", match_pat.operation),
+                    format!("{:?}", expected)
+                );
 
-                assert_eq!(match_pat.scope,
-                           vec![Scope::new("keyword.control.c").unwrap(),
-                                Scope::new("keyword.looping.c").unwrap()]);
+                assert_eq!(
+                    match_pat.scope,
+                    vec![
+                        Scope::new("keyword.control.c").unwrap(),
+                        Scope::new("keyword.looping.c").unwrap()
+                    ]
+                );
 
                 assert!(match_pat.with_prototype.is_some());
             }
@@ -975,7 +1033,8 @@ mod tests {
                   pop: true
         "#,false, None).unwrap();
 
-        let def_with_embed = SyntaxDefinition::load_from_str(r#"
+        let mut def_with_embed = SyntaxDefinition::load_from_str(
+            r#"
         name: C
         scope: source.c
         file_extensions: [c, h]
@@ -989,14 +1048,36 @@ mod tests {
               embed: scope:source.css
               embed_scope: source.css.embedded.html
               escape: (?i)(?=</style)
-        "#,false, None).unwrap();
+        "#,
+            false,
+            None,
+        )
+        .unwrap();
+
+        // We will soon do an `assert_eq!()`. But there is one difference we must expect, namely
+        // that for `def_with_embed`, the value of `ContextReference::ByScope::with_escape` will be
+        // `true`, whereas for `old_def` it will be `false`. So manually adjust `with_escape` to
+        // `false` so that `assert_eq!()` will work.
+        let def_with_embed_context = def_with_embed.contexts.get_mut("main").unwrap();
+        if let Pattern::Match(ref mut match_pattern) = def_with_embed_context.patterns[0] {
+            if let MatchOperation::Push(ref mut context_references) = match_pattern.operation {
+                if let ContextReference::ByScope {
+                    ref mut with_escape,
+                    ..
+                } = context_references[1]
+                {
+                    *with_escape = false;
+                }
+            }
+        }
 
         assert_eq!(old_def.contexts["main"], def_with_embed.contexts["main"]);
     }
 
     #[test]
     fn errors_on_embed_without_escape() {
-        let def = SyntaxDefinition::load_from_str(r#"
+        let def = SyntaxDefinition::load_from_str(
+            r#"
         name: C
         scope: source.c
         file_extensions: [c, h]
@@ -1009,7 +1090,10 @@ mod tests {
                 1: meta.tag.style.begin.html punctuation.definition.tag.end.html
               embed: scope:source.css
               embed_scope: source.css.embedded.html
-        "#,false, None);
+        "#,
+            false,
+            None,
+        );
         assert!(def.is_err());
         match def.unwrap_err() {
             ParseSyntaxError::MissingMandatoryKey(key) => assert_eq!(key, "escape"),
@@ -1019,7 +1103,8 @@ mod tests {
 
     #[test]
     fn errors_on_regex_compile_error() {
-        let def = SyntaxDefinition::load_from_str(r#"
+        let def = SyntaxDefinition::load_from_str(
+            r#"
         name: C
         scope: source.c
         file_extensions: [test]
@@ -1027,7 +1112,10 @@ mod tests {
           main:
             - match: '[a'
               scope: keyword.name
-        "#,false, None);
+        "#,
+            false,
+            None,
+        );
         assert!(def.is_err());
         match def.unwrap_err() {
             ParseSyntaxError::RegexCompileError(ref regex, _) => assert_eq!("[a", regex),
@@ -1037,8 +1125,8 @@ mod tests {
 
     #[test]
     fn can_parse_ugly_yaml() {
-        let defn: SyntaxDefinition =
-            SyntaxDefinition::load_from_str("
+        let defn: SyntaxDefinition = SyntaxDefinition::load_from_str(
+            "
         name: LaTeX
         scope: text.tex.latex
         contexts:
@@ -1060,8 +1148,10 @@ mod tests {
             - match: '(?=\\S)'
               pop: true
         ",
-                                            false, None)
-                .unwrap();
+            false,
+            None,
+        )
+        .unwrap();
         assert_eq!(defn.name, "LaTeX");
         let top_level_scope = Scope::new("text.tex.latex").unwrap();
         assert_eq!(defn.scope, top_level_scope);
@@ -1070,7 +1160,10 @@ mod tests {
         match *first_pattern {
             Pattern::Match(ref match_pat) => {
                 let m: &CaptureMapping = match_pat.captures.as_ref().expect("test failed");
-                assert_eq!(&m[0], &(1,vec![Scope::new("support.function.box.latex").unwrap()]));
+                assert_eq!(
+                    &m[0],
+                    &(1, vec![Scope::new("support.function.box.latex").unwrap()])
+                );
 
                 //use parsing::syntax_definition::ContextReference::*;
                 // TODO: check the first pushed reference is Inline(...) and has a meta_scope of meta.function.box.latex
@@ -1107,24 +1200,38 @@ mod tests {
                   escape: 'test'
             "#,
             false,
-            None
-        ).unwrap();
+            None,
+        )
+        .unwrap();
 
         assert_eq!(def.contexts["a"].meta_scope, vec![Scope::new("a").unwrap()]);
-        assert_eq!(def.contexts["#anon_a_0"].meta_scope, vec![Scope::new("anonymous_x").unwrap()]);
-        assert_eq!(def.contexts["#anon_a_1"].meta_scope, vec![Scope::new("anonymous_x_2").unwrap()]);
-        assert_eq!(def.contexts["#anon_a_2"].meta_scope, vec![Scope::new("anonymous_y").unwrap()]);
+        assert_eq!(
+            def.contexts["#anon_a_0"].meta_scope,
+            vec![Scope::new("anonymous_x").unwrap()]
+        );
+        assert_eq!(
+            def.contexts["#anon_a_1"].meta_scope,
+            vec![Scope::new("anonymous_x_2").unwrap()]
+        );
+        assert_eq!(
+            def.contexts["#anon_a_2"].meta_scope,
+            vec![Scope::new("anonymous_y").unwrap()]
+        );
         assert_eq!(def.contexts["#anon_a_3"].patterns.len(), 1); // escape
     }
 
     #[test]
     fn can_use_fallback_name() {
-        let def = SyntaxDefinition::load_from_str(r#"
+        let def = SyntaxDefinition::load_from_str(
+            r#"
         scope: source.c
         contexts:
           main:
             - match: ''
-        "#,false, Some("C"));
+        "#,
+            false,
+            Some("C"),
+        );
         assert_eq!(def.unwrap().name, "C");
     }
 
